/* --- atmcorr_.c --- */

/* 30/10/02
   SNM
   Function to estimate the radiance contribution due to atmospheric
   scattering in the 3.9 micron channel
   Inputs - satellite and solar zenith angles (rad)
            s_zen = sun_z_rad
	    o_zen = sat_z_rad
	    vis = visibility (km)
   Output - returns radiance contribution due to atmospheric scattering
            drad = radiance (mW / m^2 str)
*/

#include <stdio.h>
#include <math.h>

float atmcorr_(float *s_zen, float *o_zen, float *vis)
{ 

  /* Coefficients for calculating atomspheric scattering radiance */
  /* For visibilites of 10, 23 and 50 km ({10}{23}{50}) */
  double a_cfs[3][3] =
  {
    {0.12632655, 0.86493534, -0.032430752},
    {0.054745604, 0.88800292, -0.020061740},
    {0.024171335, 0.89530608, -0.0096661910}
  };
  double a1, a2, a3;  /* variables to hold individual coefficients */
  float drad1, drad2;  /* scattering radiances at lower and upper visibilities
			  interpolated to give single radiance (drad) */
  float drad;  /* atmospheric scattering radiance */

  /* Calculate the scattering radiance for the given visibility and view
     geometry. Interpolate between the nearest two visibilities */
  if (*vis <= 10.0) /* use vis = 10 km radiance */
    {
      a1 = a_cfs[0][0];
      a2 = a_cfs[0][1];
      a3 = a_cfs[0][2];
      drad = a1 * (1.0/cos(*s_zen)) 
	* (pow(a2,((1.0/cos(*s_zen)) + (1.0/cos(*o_zen))))) 
	+ a3;
    }

  else if (*vis > 10.0 && *vis <= 23.0)
    {
      a1 = a_cfs[0][0];
      a2 = a_cfs[0][1];
      a3 = a_cfs[0][2];
      drad1 = a1 * (1.0/cos(*s_zen)) 
	* (pow(a2,((1.0/cos(*s_zen)) + (1.0/cos(*o_zen))))) 
	+ a3;  /* radiance at vis = 10 km */
      a1 = a_cfs[1][0];
      a2 = a_cfs[1][1];
      a3 = a_cfs[1][2];
      drad2 = a1 * (1.0/cos(*s_zen)) 
	* (pow(a2,((1.0/cos(*s_zen)) + (1.0/cos(*o_zen))))) 
	+ a3; /* radiance at vis = 23 km */
      /* interpolate to give radianace at given vis */
      drad = 1.0 / (
	  (((*vis - 10.0)/(13.0)) * (1.0 / drad2))
	+ (((23.0- *vis)/(13.0)) * (1.0 / drad1))
	  );
    }

  else if (*vis > 23.0 && *vis < 50.0)
    {
      a1 = a_cfs[1][0];
      a2 = a_cfs[1][1];
      a3 = a_cfs[1][2];
      drad1 = a1 * (1.0/cos(*s_zen)) 
	* (pow(a2,((1.0/cos(*s_zen)) + (1.0/cos(*o_zen))))) 
	+ a3; /* radiance at vis = 23 km */
      a1 = a_cfs[2][0];
      a2 = a_cfs[2][1];
      a3 = a_cfs[2][2];
      drad2 = a1 * (1.0/cos(*s_zen)) 
	* (pow(a2,((1.0/cos(*s_zen)) + (1.0/cos(*o_zen))))) 
	+ a3; /* radiance at vis = 50 km */
      /* interpolate to give radianace at given vis */
      drad = 1.0 / (
	  (((*vis - 23.0)/(27.0)) * (1.0 / drad2))
	+ (((50.0- *vis)/(27.0)) * (1.0 / drad1))
	  );
    }

  else if (*vis >= 50.0) /* use vis = 50 km radiance */
    {
      a1 = a_cfs[2][0];
      a2 = a_cfs[2][1];
      a3 = a_cfs[2][2];
      drad = a1 * (1.0/cos(*s_zen)) 
	* (pow(a2,((1.0/cos(*s_zen)) + (1.0/cos(*o_zen))))) 
	+ a3;
    }


  /* return estimate of atmospheric scattering radiance */

  return drad;

}
