c	***************************************************************
	subroutine nav_pixab3(image,twilight,line,elem, scene, sat_z, 
     & sun_z, orglat, orglon, pxlat, pxlon)

c	subroutine nav_pixab2(image,twilight,line,elem, scene,sat_z,sun_z)
c	subroutine nav_sst_line( image, xlin, ierr, kdbug )
c	***************************************************************
c
c  PURPOSE:
c	Navigate the SST image
c
c  BACKGROUND:
c	Scene type is based on 1-km land mask data set by Daniel R. Steinwand 
c	(via Richard Frey) which consists of seven types of surface:
c		0: Shallow ocean (<5 km from coast or <50 m deep);
c		1: Land;
c		2: Ocean coast;
c		3: Shallow inland water;
c		4: Ephemeral water;
c		5: Deep inland water;
c		6: Deep ocean.
c
c	Solar zenith angle is computed using standard McIDAS routine.
c	In addition, during daytime, shallow water and sun glint are marked
c	as twilight zone, indicating that albedo should not be used.
c
c  METHOD:
c	If a new image - Run nvxini,solarp to find navigation constants.
c	If a new line  - Run navline to find lat/lon array for line of pixels.
c	Read lat/lon from vector, find landtype, compute sunangles
c
c  HISTORY:
c    97/12/04 Done.
c    98/02/06 1) Glint/coast are marked by negative SZA, instead of 90.
c	      2) No distinction between water and coast pixels at night.
c    00/04/25 Re-coded.
c    00/09/11 Invoke navline - reduces calls to nvxsae 80%.
c    02/05/03 SNM - modified to return pixel and satellite lat/lon
c
c  I/O LIST:
	implicit	none
	integer		image		! IN  area file of image to be navigated
	integer		twilight	! IN  solzr zenith for dawn/evening
	integer		line, elem	! IN  pixel image coordinates
	integer		scene		! OUT scene type.
					!     0: Space;
					!     1: Land;
					!     2: Ocean coast;
					!     3: Ephemeral water;
					!     4: Shallow inland water;
					!     5: Shallow ocean
					!     6: Deep inland water;
					!     7: Deep ocean.
	integer		sat_z		! OUT (sec(sat. zenith angle)-1)*1000
	integer		sun_z		! OUT solar zenith angle (0-180)
c
c  LOCAL VARIABLES:
	integer		nvset, nv1sae, nv1opt, hd(64), kk
	real		a2r, sline, selem, s_lat, s_lon, xin(4), angles(3)
	parameter	(a2r=3.14159265/180)

	character*50	topo_file            	! land mask file name
	real*8		dline, delem		! line/elem in Goode
	byte		one_byte(4)		! ORA can read 4-byte word only
	integer		conv_sce(0:6)               	! Conv. to new format
	integer		prev_img
	integer		prev_line
	integer		prev_elem

	real 		sunlat, sunlon
	real 		satlat, satlon
	real 		xsum, ysum, zsum, rsum

        integer*4       numels           ! number of elements in image line
        real*4          elnum            ! 1st pixel elem num, in Sat coords
        integer*4       elres            ! pixel sampling stepsize, Sat coords
        integer*4       lnnum            !    ??
        integer*4       lnres            !    ??
        integer*4       ierr             ! error return for navline
        logical         kdbug            ! flag for debug output 
        real*4          xlin             ! real(line)
        logical         kdbug8,kdbug9    ! extraflag for debug output 
c SNM 02/05/03
        real         orglat, orglon  ! satellite lat and lon
        real         pxlat, pxlon    ! pixel lat and lon
c       >>> [-90,90] N+ve, [0,360] westward <<<
c SNM


c  COMMON BLOCK & DEFINE VECTORS FOR NAV RESULTS

        include     "geog.f"

        save numels, elres, elnum, lnnum, lnres
        save prev_img, prev_line, prev_elem
        save s_lat, s_lon
        save xin

	data  		topo_file/'../data/lst1km.v3'/	
	data  		conv_sce/5,1,2,4,3,6,7/
	data   		prev_img/-1/
	data   		prev_line/-1/
	data   		prev_elem/-1/

        data kdbug8 /.true./

c===========================================================================

c	-------------------------------------
c	Preparation - Nav setup for new image
c	-------------------------------------
	if (prev_img .ne. image) then		! new image

c			>>>  Read the header of the image to be navigated  <<<
	   call readd(image,hd)

c			>>>  Extract parms needed for navline
           lnnum  = hd( 6)
           elnum  = float( hd( 7))
           numels = hd(10)
           lnres  = hd(12)
           elres  = hd(13)

c			>>>  Set up navigation  <<<
	   if (nvset('AREA',image) .ne. 0) stop 'NVSET in solar_zen'

c			>>>  Sub-satellite position  <<<
	   if (nv1opt('SPOS',xin, angles).ne.0) stop 'NV1OPT - SPOS'
c		"xin": not used
c		"angles": (1) & (2) are lat/lon of sub-satellite point,
c			  lon is west positive.

c			>>>  Sub-sun position  <<<
cccc	   call solarq(hd(4),hd(5), xin(3),xin(4),s_lat,s_lon)
	   call solarp(hd(4),hd(5), xin(3),xin(4),s_lat,s_lon)
c		From YYDDD & HHMMSS, "solarp" calculates UTC hour, 
c		Declination angle, and lat/lon of sub-sun point,
c		lon is 0-360 degree.

c			>>>  Mid-point between satellite and sun  <<<
c			     Computed using vector components

	   sunlat = a2r * s_lat
	   sunlon = a2r * s_lon
	   satlat = a2r * angles(2)
	   satlon = a2r * angles(2)

c       >>> satellite latitude and longitude <<<
c       >>> [-90,90] N+ve, [0,360] westward <<<
	   orglat = angles(1)
	   orglon = angles(2)

	   xsum	= cos(sunlat) * cos(sunlon) + cos(satlat) * cos(satlon)
	   ysum	= cos(sunlat) * sin(sunlon) + cos(satlat) * sin(satlon)
	   zsum	= sin(sunlat)               + sin(sunlon)
	   rsum = sqrt( xsum**2 + ysum**2 + zsum**2 )

           s_lat = asin( zsum/rsum )	/ a2r
           s_lon = atan2( ysum, xsum )	/ a2r

cccc	   s_lat = (angles(1)+s_lat)/2
cccc	   s_lon = (angles(2)-s_lon+360)/2


c			>>>  Put day/time in xin for nv1opt  <<<
	   xin(1) = hd(3)*1.e5 + mod(hd(4),100000)	! SSYYDDD and ...
	   xin(2) = hd(5)/10000+ mod(hd(5),10000)/6000.	! hour used in nv1opt


c			>>>  Open topography file  <<<
	   close(67)
c	   for Linux, recl=4
	   open (67, file=topo_file, status='old', access='direct',
     &		form='unformatted', recl=4)
	   prev_img = image

           prev_line = -1

	end if


c	End: Preparation - Nav setup for new image
c===========================================================================
c	-------------------------------------
c	If new line, navigate line of data...
c	-------------------------------------

        if (prev_line .ne. line)    then

            xlin = lnnum + (line-1) * lnres

            call navline( xlin, elnum, elres, numels, ierr, kdbug )

c            if (mod(line,100).eq.0) then
c              	print*,"new line - call navline",line
c		print*,"elnum,elres,numels,ierr: ",elnum,elres,numels,ierr
c            endif

            prev_line = line

        endif

c===========================================================================
c       If it's a new pixel, get lat-lon and scene type.
c       If a previously processed pixel, compute the sat/sun angles 
c       
        if (prev_elem .ne. elem)    then	! new pixel - get latlon

c	   ----------
c	   Navigation
c	   ----------

c			>>>  lat/lon for the pixel  <<<
           xin(3) = splat(elem)
           xin(4) = splon(elem)

c SNM
c       >>> pixel latitude and longitude <<<
c       >>> [-90,90] N+ve, [0,360] westward <<<
	   pxlat = xin(3)
	   pxlon = xin(4)
c SNM
           prev_elem = elem

 	   if (kstat(elem) .ne.0 )  then	! space
	      scene = 0
	      return
	   end if


c   	   -----
c	   Scene
c	   -----
c			>>>  Scene type for this pixel  <<<
	   xin(4) = -xin(4)	! West/East pos. for nv1sae/nv1opt
           call getcoord(dble(xin(3)),dble(xin(4)), dline,delem)	! Goode
           kk = nint(dline-1)*40031 + nint(delem)	! corresponding byte
           read (67, rec=kk/4+1) one_byte		! Read 4 bytes ...
           kk = mod(kk-1,4) + 1				! find the right one
	   scene = conv_sce(one_byte(kk))		! conv. to new format
c	   return


        else					! old pixel - get sun angles


ccc	   if (scene .lt. 4) return             ! in case main forgot to check
c	   ----------------
c	   Viewing geometry
c	   ----------------
c			>>>  Compute the angles  <<<
	   if (nv1opt('ANG ',xin, angles).ne.0) stop 'NV1OPT'


	   sat_z = nint((1./cos(angles(1)*a2r)-1)*1.e3)
	   sun_z = nint(angles(2))			! sun  zenith (0-180)

c			>>>  Reserve "twilight" for special region  <<<
	   if (sun_z .eq. twilight) then
	   sun_z = twilight + 1


c	   -------------------------------------------------
c	   Identify sunglint and coast for special treatment
c	   -------------------------------------------------
c			>>>  Coast  <<<
	   else if (sun_z .lt. twilight) then
	      if (scene.gt.3 .and. scene.lt.6) then
      	         sun_z = twilight

c			>>>  Sunglint  <<<
c	   	   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c		   May et al (I think) used abs(sun_z-sat_z)<30 degree and 
c		   azimuth<30 degree to define sunglint. 
c
c		   I found 25 is sufficient and, unlike our experience at 
c		   surface, threshold for zenith angle difference does not 
c		   seem to increase with zenith angle. 
c
c		   However, this scheme doesn't work for back-scattering, 
c		   which occur when the sun is almost behind the satellite.
c
c		   So we define the area of possible sunglint as a circle 
c		   about 10 degree from the satellite-sun middle point.
c
c		   Formula is approximate: 
c			cos(angle)=cos(dlat)cos(dlon-sin(dlat)*sin(dlon)
c			assumes dlat, dlon "small",  lat "small"
c		   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	      else if (cos((xin(3)-s_lat)*a2r)*
     *		    cos((xin(4)+s_lon)*a2r) .gt. 0.98) then
      	         sun_z = twilight
	      end if
	   end if
	end if
	return
	end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
