      subroutine navline( xlin, elnum, elres, numels, ierr, kdbug )
C========================================================================
C
C  Name- navline
C
C  Language- fortran 77  type- subroutine
C
C  Version- 1.0    date- 05/22/99   programmer- r.potash (prc)
C
C  Purpose:- Navigate single scan line of goes imager data, efficiently.
C       Since navigating every point separately costs too many cycles, 
C       use interpolation to reduce the number of calls to the navigation
C       subroutine nv1sae.  
C  Method:- For short lines, just navigate every point.  Else:
C       1. Determine range of points on earth by call to findends
C       2. Navigate the first few points individually.
C       3. Process segments: Use quadratic interpolation, calling nav subroutine
C          only center and ends of segments.   Segment size increases gradually.
C       4. Perform nav from left past center, then repeat starting from right.
C
C  Results:
C      The arrays containing lats, lons, and on-earth flag are returned 
C      through the common block geog,  defined in the include file geog.f
C
C  Assumptions:
C       Assumes navigation setup (nvxini or nxsetz) has already been performed
C
C  Subroutines called: nv1sae, findends
C  
C
C  Modification history
C    Date       Who       What
C  ---------  --------  ----------------------------------------------
C
C
C========================================================================
C
C...  DECLARATIONS

      implicit none

C  Include files

      include "geog.f"                  ! Define & Pass Results Vectors

C...Map parms, functions, subroutine args

      real*4       xlin                 ! line number - satellite coords
      real*4       elnum                ! image ULC column - satellite coords
      integer*4    elres                ! element resolution - satellite pixels
      integer*4    numels               ! number of elements in image
      integer*4    ierr                 ! error return from navline
      logical      kdbug                ! add debug output
      integer*4    nv1sae               ! (McIDAS) navigation subroutine

C...declare internal variables.

      integer*4    MAXINCREMENT         ! maximum stepsize, image pixels
      integer*4    NUMSINGLE            ! How many single pts to nav at start
      integer*4    NUMSHORT             ! Short intervals - navigate all pts
      integer*4    NUMDOUBLE            ! Num repeats between interval doubling

      integer*4    increment, Imiddle, ipass, Itest
      integer*4    j, k
      integer*4    Igood1, Igood2       ! Leftmost/Rightmost pixel on earth

      integer*4    kstat1, kstat2, kstat3
      integer*4    J1,     J2,     J3
      real*4       splon1, splon2, splon3
      real*4       splat1, splat2, splat3
      real*4       xele1,  xele2,  xele3

      real*4       difflon, difflat, diffdist !  For debug/qc 
      real*4       SMALLDIST                  !  For debug/qc 
      real*4       frac                       !  Interpolation position
      real*4       xele, z

      real*4       dlon2f, dlat2f, dlon1, dlat1 ! Coeffs for quadradic interp

      data MAXINCREMENT /   64  /  ! Longest interpolation interval
c     data MAXINCREMENT /    4  /  ! Longest interpolation interval
      data NUMSINGLE    /    4  /  ! #Points to navigate at scanline ends
      data NUMSHORT     /  100  /  ! If scanline shorter, navigate all points
      data SMALLDIST    / 0.125 /  ! Error threshhold - not used currently 
      data NUMDOUBLE    /  1    /  ! #Repeats at each interval size-then double
       
C========================================================================
C...  BEGIN EXECUTION

      if (kdbug) write (6,*)  'navline4d: kdbug=.true.'

C     Initialize all points, return code to bad. 

      ierr = -1
      do j = 1, numels
         kstat(j) =  -1
      enddo


C...  For shorter lines, just navigate every point and go home.
      if (numels .lt. NUMSHORT) then     
         write (6,*)  'Short Line.',numels,NUMSHORT
         do j = 1, numels
            xele  = elnum + (j-1) * elres
            kstat(j) = nv1sae(xlin,xele,0.,splat(j),splon(j),z)
         end do
         ierr = 0
         goto 99
      end if

C     Find leftmost and rightmost point on earth

      call findends(xlin,elnum,elres,numels, Igood1,Igood2,ierr)

ccc   if (kdbug) then
ccc      write (6,*)  'findends: Leftmost point  =',Igood1
ccc      write (6,*)  'findends: Rightmost point =',Igood2
ccc   endif

      if (Igood1 .le.0 .or. Igood2 .le. 0)   then
         ierr = 1
         goto 88
      end if

C...  For shorter lines (on earth), navigate every point and go home.
      if (Igood2 - Igood1  .lt. NUMSHORT) then     
         write (6,*)  'Short Line earth intersection.',numels,NUMSHORT
         do j = Igood1, Igood2
            xele  = elnum + (j-1) * elres
            kstat(j) = nv1sae(xlin,xele,0.,splat(j),splon(j),z)
         end do
         ierr = 0
         goto 99
      end if

C--------------------------------------------------------------------
C...     Long stretch of data.   Navigate in segments.

         Imiddle = (Igood1 + Igood2) / 2

C...     Begin processing left half of scanline, from edge inward
         J3 = Igood1 + NUMSINGLE
         increment = 1
         ipass = 0

C        Navigate the first few points individually
         do j = Igood1, J3
            xele  = elnum + (j-1) * elres
            kstat(j) = nv1sae(xlin,xele,0.,splat(j),splon(j),z)
         end do

C        Navigate segments of line using quadratic interpolation
         do while (J3 .lt. Imiddle)
C           Decide whether to increase segment length
            ipass = ipass + 1
            if (increment .lt. MAXINCREMENT)              then
               if (mod(ipass,NUMDOUBLE) .eq. 0)           then
                  if ( J3 + 2 * increment .lt. Imiddle )  then
                     increment = increment * 2
                     Itest=J1
                  end if
               end if
ccc            if (kdbug) write(6,*) ' navline2/increment=',increment
            end if
            J1 = J3
            J2 = J1 + increment
            J3 = J1 + increment * 2
ccc         if (kdbug) write(6,*) ' navline2/left half - J1= ',J1
            xele1 = elnum + elres * (J1-1)
            xele2 = elnum + elres * (J2-1)
            xele3 = elnum + elres * (J3-1)
            kstat1 = nv1sae(xlin,xele1,0.,splat1,splon1,z)
            kstat2 = nv1sae(xlin,xele2,0.,splat2,splon2,z)
            kstat3 = nv1sae(xlin,xele3,0.,splat3,splon3,z)
c...        Fix any branch cuts
            if (abs(splon1-splon3) .gt. 90.0)  then
               splon1 = amod( splon1 + 360., 360. )
               splon2 = amod( splon2 + 360., 360. )
               splon3 = amod( splon3 + 360., 360. )
            endif
            dlat1 = ( splat3 - splat1 ) / 2.0
            dlon1 = ( splon3 - splon1 ) / 2.0
            dlat2f= ( splat3 - 2. * splat2 + splat1 ) / 2.0
            dlon2f= ( splon3 - 2. * splon2 + splon1 ) / 2.0
            do k = -Increment, Increment
               frac = float(k)/float(Increment)
               splat(J2+k) = splat2 + dlat1 * frac + dlat2f * frac**2
               splon(J2+k) = splon2 + dlon1 * frac + dlon2f * frac**2
               kstat(J2+k) = 0
            enddo
         enddo
C...     Done processing left half of scanline.

C...     Begin processing right half of scanline, from edge inward
         J1 = Igood2 - NUMSINGLE
         increment = 1
         ipass = 0

C        Navigate the last few points individually
         do j = Igood2, J1, -1
            xele  = elnum + (j-1) * elres
            kstat(j) = nv1sae(xlin,xele,0.,splat(j),splon(j),z)
         end do

C        Navigate segments of line using quadratic interpolation
         do while (J1 .gt. Imiddle)
C           Decide whether to increase segment length
            ipass = ipass + 1
            if (increment .lt. MAXINCREMENT)              then
               if (mod(ipass,NUMDOUBLE) .eq. 0)           then
                  if ( J1 - 2 * increment .gt. Imiddle )  then
                     increment = increment * 2
                  end if
               end if
ccc            if (kdbug) write(6,*) ' navline2/increment=',increment
            end if

            J3 = J1
            J2 = J3 - increment
            J1 = J3 - increment * 2
ccc         if (kdbug) write(6,*) ' navline2/right half - J3= ',J3
            xele1 = elnum + elres * (J1-1)
            xele2 = elnum + elres * (J2-1)
            xele3 = elnum + elres * (J3-1)
            kstat1 = nv1sae(xlin,xele1,0.,splat1,splon1,z)
            kstat2 = nv1sae(xlin,xele2,0.,splat2,splon2,z)
            kstat3 = nv1sae(xlin,xele3,0.,splat3,splon3,z)
c           Fix any branch cuts
            if (abs(splon1-splon3) .gt. 90.0)  then
               splon1 = amod( splon1 + 360., 360. )
               splon2 = amod( splon2 + 360., 360. )
               splon3 = amod( splon3 + 360., 360. )
            endif
            dlat1 = ( splat3 - splat1 ) / 2.0
            dlon1 = ( splon3 - splon1 ) / 2.0
            dlat2f= ( splat3 - 2. * splat2 + splat1 ) / 2.0
            dlon2f= ( splon3 - 2. * splon2 + splon1 ) / 2.0
            do k = -Increment, Increment
               frac = float(k)/float(Increment)
               splat(J2+k) = splat2 + dlat1 * frac + dlat2f * frac**2
               splon(J2+k) = splon2 + dlon1 * frac + dlon2f * frac**2
               kstat(J2+k) = 0
            enddo
         enddo
C...     Done processing right half of scanline.

C        Debug and QC output
         if (kdbug) then
            write(6,*) ' navline2 - check some points'
            write(6,*) ' Igood1, Increment ', Igood1, Increment
            write(6,*) ' Igood2, Increment ', Igood2, Increment

            write(6,*) ' '
            do j=Igood1, Igood1 + Increment * 4, 1
               xele = elnum + elres * (j-1)
               kstat1 = nv1sae(xlin,xele,0.,splat1,splon1,z)
               difflon = (splon(j) - splon1) * cos(splat1/57.296)
               difflat = (splat(j) - splat1)
               diffdist =  100.0 * sqrt( difflat**2 + difflon**2 ) 
               if (diffdist .gt. SMALLDIST) then
               write(6,1000) j,splat1,splon1,difflon,difflat,diffdist
               endif
 1000          format(' j,lat,lon,diflon/lat/d-km: ',i4,2x, 2f7.2,
     *                    2x, 2f7.2, f11.2)
            enddo

            write(6,*) ' '
            do j=Igood2, Igood2 - Increment * 4, -1
               xele = elnum + elres * (j-1)
               kstat1 = nv1sae(xlin,xele,0.,splat1,splon1,z)
               difflon = (splon(j) - splon1) * cos(splat1/57.296)
               difflat = (splat(j) - splat1)
               diffdist =  100.0 * sqrt( difflat**2 + difflon**2 ) 
               if (diffdist .gt. SMALLDIST) then
               write(6,1000) j,splat1,splon1,difflon,difflat,diffdist
               endif
            enddo
            write(6,*) ' '

            do j = Itest, Igood2, increment/4 
               xele = elnum + elres * (j-1)
               kstat1 = nv1sae(xlin,xele,0.,splat1,splon1,z)
               difflon = (splon(j) - splon1) * cos(splat1/57.296)
               difflat = (splat(j) - splat1)
               diffdist =  100.0 * sqrt( difflat**2 + difflon**2 ) 
               if (diffdist .gt. SMALLDIST) then
               write(6,1000) j,splat1,splon1,difflon,difflat,diffdist
               endif
            enddo
            write(6,*) ' '

         endif

      ierr = 0

   88 continue
   99 continue

      return
      end

C...  End of subroutine navline
C========================================================================


      subroutine findends(xlin,elnum,elres,numels, Igood1,Igood2,ierr)

C========================================================================
C 
C  Name- findends
C
C  Language- fortran 77  type- subroutine
C
C  Version- 1.0 date- 05/22/99   programmer- r.potash (prc)
C
C  Purpose:- For goes imager linescan, find left & right most points on earth
C
C  Method:- Check if either end points off-earth.
C    If both ends points are off earth, try several to find one
C    on earth.  Then use binary search to find the actual end points.  
C
C  Subroutines called: nv1sae
C
C  Modification history
C  Date       Who       What
C  ---------  --------  ----------------------------------------------
C  99May26    R.Potash  Initial version
C
C========================================================================
C
C...  DECLARATIONS

      implicit none

C...declare nonlocal variables

      integer*4    Igood1, Igood2       ! Leftmost/Rightmost pixel on earth
      real*4       xlin, elnum
      integer*4    elres
      integer*4    numels, ierr
      integer*4    nv1sae               ! (McIDAS) navigation subroutine
      integer*4    kstatj
      real*4       splonj, splatj

C...declare local variables.

      logical      kbad1, kbad2         ! Flag - 1st/last pixel in space
      logical      kdbug,kdbug2

      integer*4    NUMTRY

      integer*4    j, jj, k, kk
      integer*4    Ibad1,  Ibad2        ! Rightmost/Leftmost pixel in space
      integer*4    IXgood               ! One point on earth 

      integer*4    J1,     J2,     J3

      real*4       xele, z

      data NUMTRY       /   6 /
    
C========================================================================
C...  BEGIN EXECUTION

      kdbug = .true.

c      kdbug2 = (  kdbug   .and.    mod(xlin,100) .lt. 1  )
c      if (kdbug2)    then
c        write(6,*) 'findends: xlin=',xlin
c        write(6,*) 'findends: elnum,elres,numels=',elnum,elres,numels
c      endif

C...  Initialize return code to bad

      Igood1 = 0
      Igood2 = 0
      ierr   = -1


C...  Find out if first and last points are good or bad

      j = 1
      xele  = elnum + (j-1) * elres
      kstatj = nv1sae(xlin,xele,0.,splatj,splonj,z)
      if (kstatj .eq. 0)  then
         kbad1 = .false.
         Igood1 = j
         IXgood = j
      else
         kbad1 = .true.
         Ibad1  = j
      endif

c      if (kdbug2) write(6,*) 'xlin,xele,kstatj    =',xlin,xele,kstatj
c      if (kdbug2) write(6,*) 'splatj,splonj=',splatj,splonj


        
      j = numels
      xele  = elnum + (J-1) * elres
      kstatj = nv1sae(xlin,xele,0.,splatj,splonj,z)
      if (kstatj .eq. 0)  then
         kbad2 = .false.
         Igood2 = j
         IXgood = j
      else
         kbad2 = .true.
         Ibad2  = j
      endif

c      if (kdbug2) write(6,*) 'xlin,xele,kstatj    =',xlin,xele,kstatj
c      if (kdbug2) write(6,*) 'splatj,splonj=',splatj,splonj

C... If both points beyond the edge of the earth, search to
C... decide if there are any good points at all.
C... Quick and dirty search

      if ( kbad1 .and. kbad2 )   then
         do kk=1,NUMTRY
            j = ( numels * kk ) / (NUMTRY+1)
            xele  = elnum + (J-1) * elres
            kstatj = nv1sae(xlin,xele,0.,splatj,splonj,z)
c      if (kdbug2) write(6,*) 'xlin,xele,kstatj    =',xlin,xele,kstatj
c      if (kdbug2) write(6,*) 'splatj,splonj=',splatj,splonj
            if (kstatj .eq. 0)  then        ! If Found a good point
                IXgood=j
                goto 30                       ! Then Jump out of loop
            end if
         enddo
C...     If reach here, no good points found. => give up on this line.
         ierr = 1
         goto 88
      endif
C...  Jump here when a good point is found
30    continue


C...  If left image edge not on earth, search for FIRST good point

      if ( kbad1 ) then
         Igood1 = IXgood
         if (Igood1 - Ibad1 .le. 0)  goto 88    !*** Cant't happen ***

C...     Binary Search for first good point
         do while (Igood1-Ibad1 .gt. 1)
           j = (Igood1 + Ibad1) / 2
           xele  = elnum + (j-1) * elres
           kstatj = nv1sae(xlin,xele,0.,splatj,splonj,z)
c      if (kdbug2) write(6,*) 'xlin,xele,kstatj    =',xlin,xele,kstatj
c      if (kdbug2) write(6,*) 'splatj,splonj=',splatj,splonj
           if (kstatj .eq. 0)  then
              Igood1 = j
           else
              Ibad1 = j
           endif
         enddo
C...     End - Binary Search for first good point

      endif


C...  If right image edge not on earth, search for LAST good point

      if ( kbad2 ) then
         Igood2 = IXgood
         if (Ibad2-Igood2 .le. 0)  goto 88    !*** Cant't happen ***

C...     Binary Search for last good point
         do while (Ibad2-Igood2 .gt. 1)
           j = (Ibad2 + Igood2) / 2
           xele  = elnum + (J-1) * elres
           kstatj = nv1sae(xlin,xele,0.,splatj,splonj,z)
c      if (kdbug2) write(6,*) 'xlin,xele,kstatj    =',xlin,xele,kstatj
c      if (kdbug2) write(6,*) 'splatj,splonj=',splatj,splonj
           if (kstatj .eq. 0)  then
              Igood2 = j
           else
              Ibad2 = j
           endif
         enddo
C...     End - Binary Search for last good point

      endif
ccc   if (kdbug) write (6,*)  'findends: Leftmost point  =',Igood1
ccc   if (kdbug) write (6,*)  'findends: Rightmost point =',Igood2
c      if (kdbug2) write (6,*)  'findends: Leftmost point  =',Igood1
c      if (kdbug2) write (6,*)  'findends: Rightmost point =',Igood2


   88 continue
      return
      end

C...  End of subroutine findends
C========================================================================
